#######################################################################
# Load Packages and set user
#######################################################################
using Distributions, PyPlot, StatsBase, DataFrames, CSV, Combinatorics, Distributed, SpecialFunctions, 
Random, LinearAlgebra, DelimitedFiles, ControlSystems
usr = "<Insert your computer username>"

#######################################################################
# Create function that returns credible interval
#######################################################################
function cred_int(X,ci)
    # X is a T by G posterior matrix containing values for one parameter at each point in time across all draws.
    # 0<ci<100 is the credible interval desired.

    (T,G) = size(X)
    pctile_low = zeros(T)
    pctile_high = zeros(T)
    int = ci/2
    lower = (50-int)/100
    upper = (50+int)/100
    for i=1:T
        pctile_low[i] = quantile(X[i,:][:],lower)
        pctile_high[i] = quantile(X[i,:][:],upper)
    end
    return pctile_low, pctile_high
end


#######################################################################
# Include break model functions, change directory path, set series
# names and number of simulations.
#######################################################################
include("/Users/$(usr)/<Insert rest of path to folder holding Julia files>/break_model_cI_prior.jl")
cd("/Users/$(usr)/<Insert rest of path to folder holding raw data>/")
varNames = ["AHETPI";"CPILFESL";"PCEPILFE";"FEDFUNDS";"GDPC1";"GPDIC1";
    "PCECC96";"PAYEMS";"UNRATE"]
casualNames = ["Hourly Earnings Growth";
               "Core CPI Inflation";
               "Core PCE Inflation";
               "Federal Funds Rate";
               "Real GDP Growth";             
               "Real Investment Growth";
               "Real PCE Growth";
               "Employment Growth";
               "Unemployment Rate"];

M = 40000;
mm1 = Int64(round(M./20));


#######################################################################
# Plot all series on one figure
# This figure is not included in the paper.
#######################################################################
figure(figsize=(8,8))
for i=1:9
    varName = varNames[i]
    Y = convert(Array{Float64,1},readdlm("Break_data/$(varName).csv",',')[2:end,2])
    T = length(Y)

    start_date = 1959.0
    end_date = 2018.25
    dates = collect(start_date:0.25:end_date)
    Tfull = 238
    dates = dates[(Tfull-T+1):end]



	if i==1
		subplot2grid((5,4), (0,0), colspan=2)
	elseif i==2
		subplot2grid((5,4), (0,2), colspan=2)
	elseif i==3
		subplot2grid((5,4), (1,0), colspan=2)
	elseif i==4
		subplot2grid((5,4), (1,2), colspan=2)
	elseif i==5 
        subplot2grid((5,4), (2,0), colspan=2)
		#subplot2grid((3,4), (2,1), colspan=2)
    elseif i==6
        subplot2grid((5,4), (2,2), colspan=2)
    elseif i==7
        subplot2grid((5,4), (3,0), colspan=2)
    elseif i==8
        subplot2grid((5,4), (3,2), colspan=2)
    elseif i==9
        subplot2grid((5,4), (4,1), colspan=2)
	end
    
	title("$(varName)")
	plot(dates,Y)
end
tight_layout()


#######################################################################
# Set up simulation framework and retrieve information from posterior
# draws for all series
#######################################################################
num_series = length(varNames)
k_pot=6
T = 237
T_vec = zeros(num_series)
start_vec = zeros(num_series)
end_vec = zeros(num_series)

B_sampled_meta = zeros(k_pot+1,T,M,num_series)
H_sampled_meta = zeros(T,M,num_series)
AR_sampled_meta = zeros(T,M,num_series)
Y_hat_sampled_meta = zeros(T,M,num_series)
x_sampled_meta = zeros(k_pot,M,num_series)
q_sampled_meta = zeros(k_pot+2,T,M,num_series)

for unique_name=1:num_series
    series_name = varNames[unique_name]
    Y = CSV.read("Break_data/$(series_name).csv")
    if (series_name=="no_break") || (series_name=="int_small")
        Y = Y[:,2]
        Y = convert(Array{Float64,1},Y)
    else
        Y = convert(Array{Float64,1},Y[:,2])
    end
    T = length(Y)

    start_date = 1959.0
    end_date = 2020.0
    dates = collect(start_date:0.25:end_date)
    dates = dates[1:T]
    end_date = dates[end]

    xj_total = readdlm("Storage/$(series_name)/xj.csv",',')
    (lags,G) = size(xj_total)
    xj_pre = readdlm("Storage/$(series_name)/xj_pre.csv",',')
    G0 = size(xj_pre,2)
    seeds = readdlm("Storage/$(series_name)/seeds.csv",',')
    num_cores = length(seeds)

    # Create potential X Variables
    lags = 6 # Maximum number of lags to include in the regression.
    x_pot = Y[1:end-lags]
    for i=lags-1:-1:1
        j = lags-i+1
        x_pot = [Y[j:end-i] x_pot]
    end

    # Redefine Y and T after including lags.
    Y = Y[lags+1:end];
    T = length(Y)

    ########################################################################
    # Set Model Priors
    # Note: I set "max_m=10" to prevent over-fitting.
    ########################################################################
    k_pot = size(x_pot,2)
    k_pot = convert(Int64,k_pot)

    minp = 8
    num_tau = T-(2*minp)+1
    num_tau = convert(Int64,num_tau)

    max_m = floor(T/minp) .- 1
    max_m = convert(Int64,max_m)
    max_m = 10

    mmm_vec = collect(0:1:max_m)
    tau_prior =  (1 ./ (exp.(lgamma.(num_tau .- (mmm_vec .- 1).*(minp) .+ 1) - 
                lgamma.(mmm_vec .+ 1) - lgamma.(num_tau .- (mmm_vec .- 1)*(minp) .- mmm_vec .+1))));

    model_size_prior = break_model_2.msp_compute(k_pot,max_m);


    ########################################################################
    # Set Parameter Priors
    ########################################################################
    # Prior on precision of variance break
    mb=1.0
    vb=0.1

    ########################################################################
    # Prior on B is N(0,V) where V is block diagonal
    ########################################################################
    # V = [1e8 ...
    #	   ... d1^2 ...
    #	   ... ...  d2^2 ...
    #	   ... ...  .... d3^2]
    # where d1 is the prior SD on full sample AR terms
    # d2 is the prior SD on intercept breaks
    # d3 is the prior SD on AR breaks

    # Common intercept prior
    int_prior_var = 1e8
    # d1,d2,d3
    c = 1 ./ mean(Chi(1))
    d2 = 2.0 ./ (c*4) # For d2 run OLS on prior observarions and set d2 = 4*(1-sum(AR))/(4c)...this implies an SD = 4% annualized
    d_vec = [4.0^2;d2.^2;4.0^2]

    ########################################################################
    # Set Other Priors
    ########################################################################
    # Prior on variance precision in all models.
    #v = 2.0; m = 5.0 # Implied SD 90% CI = [0.25,1.98]; 99% CI = [0.19,6.26]
    v = 1e-10; m = 1.0 # Noninformative prior

    names = ["Intercept"]
    for i=2:k_pot+1
        names = [names;"Variable $(i-1)"]
    end
    names = [names;"Conditional Variance"]
    
    dates = dates[lags+1:end]

    #######################################################################
    # Compute Stats using Analytical Model Probabilities
    #######################################################################
    qj_total = readdlm("Storage/$(series_name)/qj.csv",',')
    qj_total = reshape(qj_total,(k_pot+2,T,G))

    xj_total = readdlm("Storage/$(series_name)/xj.csv",',')
    # Create matrix that contains location of breaks
    qj = deepcopy(qj_total)
    xj = deepcopy(xj_total)

    # Find number of parameter breaks in each model
    l_sto = zeros(G)
    for i=1:G
        l_sto[i] = length(findall(qj[:,:,i] .!= 0))
    end
    l_max = convert(Int64,maximum(l_sto)) # Maximum number of breaks in any model
    q_place_sto = zeros(l_max,G) # Storage matrix for break dates

    for i=1:G
        l = convert(Int64,l_sto[i])
        place = findall(vec(qj[:,:,i]) .!= 0)
        if l>0
            q_place_sto[1:l,i] .= place
        end
    end
    q_place_sto = convert(Array{Int64,2},q_place_sto)
    
    #######################################################################
    # Find all unique models
    #######################################################################
    model =[1]
    q_unique = collect(q_place_sto[:,1])
    x_unique = collect(xj[:,1])
    times = [sum(sum(q_place_sto[:,1].==q_place_sto,dims=1)[:] .==l_max)]
    times = [1]

    for i=2:G
        q = q_place_sto[:,i]
        x = xj[:,i]
        if ((sum(sum(q.==q_unique,dims=1)[:] .==l_max)) == 0)
                 model =[model;model[end]+1]
                 q_unique = [q_unique q]
                 x_unique = [x_unique x]
                 times = [times;1]
        elseif ((sum(sum(q.==q_unique,dims=1)[:] .==l_max)) > 0)
            unique_vec = findall(sum(q.==q_unique,dims=1)[:] .== l_max)
            if length(unique_vec)==1
                j = findall(sum(q.==q_unique,dims=1)[:] .== l_max)[1]
                if sum(x.==x_unique[:,j],dims=1)[:][1].==k_pot
                    times[j] = times[j]+1
                else
                     model =[model;model[end]+1]
                     q_unique = [q_unique q]
                     x_unique = [x_unique x]
                     times = [times;1]
                end
            else
                itr = 0
                for k=1:length(unique_vec)
                    j = findall(sum(q.==q_unique,dims=1)[:] .== l_max)[k]
                    if sum(x.==x_unique[:,j],dims=1)[:][1].==k_pot
                        times[j] = times[j]+1
                        itr+=1
                    end
                end
                if itr==0
                     model =[model;model[end]+1]
                     q_unique = [q_unique q]
                     x_unique = [x_unique x]
                     times = [times;1]
                end
            end 
        end
    end
    probs = times./G

    # Use full list and compute analytical model probabilities
    G1 = length(probs)

    println("")
    println("Number of unique models visited by the sampler = $(G1)")
    println("")

    #######################################################################
    # Create Analytical Probabilities for All Unique Models
    #######################################################################

    k_pot = size(x_pot,2)
    ln_lik_sto = zeros(G1)

    print("Computing analytical probabilities.")
    gg1 = Int64(round(G1./20,digits=0))
    for i=1:G1
        if mod(i,gg1)==0
            print(".")
        end

        q = q_unique[:,i]
        qq = zeros(k_pot+2,T)
        qq[q[findall(q .!=0)]] .= 1.0
        qj_in = deepcopy(qq)

        xj_in = x_unique[:,i]
        k_in = convert(Int64,sum(xj_in))

        qj_reduced = qj_in[findall([1;xj_in;1] .!=0),:]
        (kk,TT) = size(qj_reduced)

        # Find marginal likelihood of current model
        num_break_param = sum(qj_reduced[1:k_in+1,:],dims=2)
        x_proc = break_model_2.make_X_break(x_pot,xj_in,qj_reduced,k_in,num_break_param,TT)
        var_break_indic = qj_reduced[k_in+2,:][:]
        if sum(var_break_indic)==0
            (ln_fy_proc,B_med,Htt_diag,flag) = break_model_2.ln_marg_lik(Y, x_proc, d_vec, int_prior_var, m, v, k_in, num_break_param);
            else
            (ln_fy_proc,B_med,Htt_diag,H_med) = break_model_2.ln_marg_lik_gs(Y, x_proc, var_break_indic, d_vec, int_prior_var, m, v, mb, vb, k_in, num_break_param);
        end
        num_param = size(x_proc,2) + sum(var_break_indic) + 1;
        num_break = Int64(length(findall(sum(qj_reduced,dims=1)[:] .!=0)))
        model_size_place = findall(model_size_prior[:,1].==num_param)[1]
        model_size_sim = model_size_prior[model_size_place,2]
        lnML_current = ln_fy_proc + log.(tau_prior[num_break+1]) + log(model_size_sim)
        ln_lik_sto[i] = lnML_current
    end
    println("")
    max_ln = maximum(ln_lik_sto)
    lik = exp.(ln_lik_sto.-max_ln)
    analytic_probs = (lik./sum(lik))

    #######################################################################
    # Use analytic probabilities to sample from model space
    #######################################################################
    x = collect(1:1:G1)
    B_sampled = zeros(k_pot+1,T,M)
    H_sampled = zeros(T,M)
    AR_sampled = zeros(T,M)
    Y_hat_sampled = zeros(T,M)
    x_sampled = zeros(lags,M)
    q_sampled = zeros(lags+2,T,M)

    print("Sampling from model space.")
    for itr=1:M
        if mod(itr,mm1)==0
            print(".")
        end

        i = sample(x,Weights(analytic_probs))
        q = q_unique[:,i]
        qq = zeros(k_pot+2,T)
        qq[q[findall(q .!=0)]] .= 1.0
        qj_in = deepcopy(qq)

        xj_in = x_unique[:,i]
        k_in = convert(Int64,sum(xj_in))

        qj_reduced = qj_in[findall([1;xj_in;1] .!=0),:]
        (kk,TT) = size(qj_reduced)

        # Find marginal likelihood of current model
        num_break_param = sum(qj_reduced[1:k_in+1,:],dims=2)
        x_proc = break_model_2.make_X_break(x_pot,xj_in,qj_reduced,k_in,num_break_param,TT)
        var_break_indic = qj_reduced[k_in+2,:][:]
        if sum(var_break_indic)==0
            (ln_fy_proc,B_med,Htt_diag,flag) = break_model_2.ln_marg_lik(Y, x_proc, d_vec, int_prior_var, m, v, k_in, num_break_param);
            else
            (ln_fy_proc,B_med,Htt_diag,H_med) = break_model_2.ln_marg_lik_gs(Y, x_proc, var_break_indic, d_vec, int_prior_var, m, v, mb, vb, k_in, num_break_param);
        end

        B_vec = break_model_2.gen_B(xj_in,k_in,qj_reduced,B_med,T,k_pot)
        H_vec = deepcopy(Htt_diag)
        AR_sum = sum(B_vec[2:end,:],dims=1)[:]
        Yhat = x_proc*B_med

        B_sampled[:,:,itr] = B_vec
        H_sampled[:,itr] = H_vec
        AR_sampled[:,itr] = AR_sum
        Y_hat_sampled[:,itr] = Yhat
        x_sampled[:,itr] = xj_in
        q_sampled[:,:,itr] = qj_in
    end
    
    B_sampled_meta[:,1:T,:,unique_name] = B_sampled
    H_sampled_meta[1:T,:,unique_name] = H_sampled
    AR_sampled_meta[1:T,:,unique_name] = AR_sampled
    Y_hat_sampled_meta[1:T,:,unique_name] = Y_hat_sampled
    x_sampled_meta[:,:,unique_name] = x_sampled
    q_sampled_meta[:,1:T,:,unique_name] = q_sampled
    T_vec[unique_name] = T
    start_vec[unique_name] = start_date
    end_vec[unique_name] = end_date
end


#######################################################################
# Calculate variable inclusion for each AR parameter for each series.
# This is table 7 in the paper.
#######################################################################
k = length(varNames)
AR_inc_mat = mean(x_sampled_meta,dims=2)
println("Inclusion probabilities for AR parameters (Table 7)")
for i=1:k
	casual_name = casualNames[i]
    println(casual_name)
    AR_inc = AR_inc_mat[:,:,i]
    n = length(AR_inc)
	for j=1:n
		println("AR Lag $(j): $(AR_inc[j])")
	end
	println("")
end


#######################################################################
# Calculate posterior intercept break probabilities for each series.
# This is part of Table 9 in the paper.
#######################################################################
println("Posterior intercept break probabilities for each series (Table 9)")
for i=1:num_series
    series_name = varNames[i]
    casual_name = casualNames[i]
    display("$(i) of $(num_series) - $(casual_name)")

    T = Int64(T_vec[i])
    qj_total = q_sampled_meta[:,1:T,:,i]
    
    int_breaks = zeros(T,M)
    for i=1:T
        for j=1:M
            if (qj_total[1,i,j]).>0
                int_breaks[i,j] = 1
            end
        end
    end
    int_break_all = sum(int_breaks,dims=1)[:]
    max_breaks = Int64(maximum(int_break_all))
    int_break_probs = zeros(max_breaks+1)
    for i=0:max_breaks
        int_break_probs[i+1] = sum(int_break_all.==i)./M
    end
    break_mat = ["Int Breaks" "Probability"; collect(0:max_breaks) int_break_probs]
    display(break_mat)
    display("")
end
println("")

#######################################################################
# Calculate posterior AR coefficient break probabilities for each series.
# This is part of Table 9 in the paper.
#######################################################################
println("Posterior AR break probabilities for each series (Table 9)")
for i=1:num_series
    series_name = varNames[i]
    casual_name = casualNames[i]
    display("$(i) of $(num_series) - $(casual_name)")
    
    T = Int64(T_vec[i])
    qj_total = q_sampled_meta[:,1:T,:,i]
    
    ar_breaks = zeros(T,M)
    for i=1:T
        for j=1:M
            if sum(qj_total[2:7,i,j]).>0
                ar_breaks[i,j] = 1
            end
        end
    end
    ar_break_all = sum(ar_breaks,dims=1)[:]
    max_breaks = Int64(maximum(ar_break_all))
    ar_break_probs = zeros(max_breaks+1)
    for i=0:max_breaks
        ar_break_probs[i+1] = sum(ar_break_all.==i)./M
    end
    break_mat = ["AR Breaks" "Probability"; collect(0:max_breaks) ar_break_probs]
    display(break_mat)
    display("")
end
println("")

#######################################################################
# Calculate posterior variance break probabilities for each series.
# This is part of Table 9 in the paper.
#######################################################################
println("Posterior variance break probabilities for each series (Table 9)")
for i=1:num_series
    series_name = varNames[i]
    casual_name = casualNames[i]
    display("$(i) of $(num_series) - $(casual_name)")

    T = Int64(T_vec[i])
    qj_total = q_sampled_meta[:,1:T,:,i]
    
    var_breaks = zeros(T,M)
    for i=1:T
        for j=1:M
            if (qj_total[end,i,j]).>0
                var_breaks[i,j] = 1
            end
        end
    end
    var_break_all = sum(var_breaks,dims=1)[:]
    max_breaks = Int64(maximum(var_break_all))
    var_break_probs = zeros(max_breaks+1)
    for i=0:max_breaks
        var_break_probs[i+1] = sum(var_break_all.==i)./M
    end
    break_mat = ["Var Breaks" "Probability"; collect(0:max_breaks) var_break_probs]
    display(break_mat)
    display("")
end
println("")

#######################################################################
# Calculate break probability by date for each variable
# This prints the numbers provided in Table 8 of the paper
#######################################################################
Tmax = Int64(maximum(T_vec))
mbreaks = zeros(Tmax,num_series)
println("Probablity of number of break dates (Table 8)")
for i=1:num_series
    series_name = varNames[i]
    casual_name = casualNames[i]
    println("$(i) of $(num_series) - $(casual_name)")

    T = Int64(T_vec[i])
    qj = q_sampled_meta[:,1:T,:,i]

    # Matrix containing all breaks
    breaks = zeros(T,M)
    for iii=1:T
        for j=1:M
            if sum(qj[:,iii,j]).>0
                breaks[iii,j] = 1
            end
        end
    end
    
    breaks_by_sim = sum(breaks,dims=1)
    max_breaks = maximum(breaks_by_sim)
    for iii=0:max_breaks
    	if iii==1
    		println("Probability of $(iii) break date  = $(mean(breaks_by_sim.==iii))")
    	else
    		println("Probability of $(iii) break dates = $(mean(breaks_by_sim.==iii))")
    	end
    end

    if T==Tmax
        mbreaks[:,i] = mean(breaks,dims=2)
    else
        mbreaks[(Tmax-T+1):end,i] = mean(breaks,dims=2)
    end
    println("")
end


#######################################################################
# Calculate intercept break probability by date for each series
#######################################################################
println("Calculating posterior intercept break probabilities by date for each series...")

mbreaks_int = zeros(Tmax,k)
for i=1:k
    series_name = varNames[i]
    casual_name = casualNames[i]
    println("$(i) of $(k) - $(casual_name)")

    T = Int64(T_vec[i])
    qj = q_sampled_meta[:,1:T,:,i]

    # Matrix containing all breaks
    breaks = zeros(T,M)
    for iii=1:T
        for j=1:M
            if sum(qj[1,iii,j]).>0
                breaks[iii,j] = 1
            end
        end
    end
    
    if T==Tmax
        mbreaks_int[:,i] = mean(breaks,dims=2)
    else
        mbreaks_int[(Tmax-T+1):end,i] = mean(breaks,dims=2)
    end
end
println("")

#######################################################################
# Create ridge plot for all intercept breaks (no FF rate)
# Figure 1 in paper
#######################################################################
dates = collect(1959:0.25:2018.25)
dates = dates[k_pot+1:end]
displace = 1.0 # 0.07
series_no_FF = [1;2;3;5;6;7;8;9]
num_series_no_FF = length(series_no_FF)
casualNamesNoFF = casualNames[[collect(1:3);collect(5:9)]]
color_vec = ["#1f77b4";"#ff7f0e";"#2ca02c";"#d62728";"#9467bd";
            "#8c564b";"#e377c2";"#7f7f7f";"#bcbd22";"#17becf"]
color_vec = [color_vec;color_vec;color_vec[1]]

figure(figsize=(10,8))
for i in series_no_FF
    if i<=3
        y = mbreaks_int[:,num_series-i+1]
    elseif (i==5) || (i==6)
        y = mbreaks_int[:,num_series-i+2]
    elseif i>6
        y = mbreaks_int[:,num_series-i+1]
    end
    #y = mbreaks[:,i]
    if i>4
        j=i-1
    else
        j=deepcopy(i)
    end
    y0 = ones(Tmax)*j*displace
    y1 = y .+ j*displace
    #k = mod(num_series,i)
    c = color_vec[i]
    fill_between(dates,y0,y1,lw=1,facecolor=c,zorder=num_series-j+1,alpha=0.7,edgecolor="k")
end
xlim([dates[1];dates[end]])
y0_vec = zeros(num_series)

for i=1:(num_series-1)
    y0_vec[i] = i*displace
    j=i-1
end
yticks(y0_vec,casualNamesNoFF[end:-1:1])

#yticks(y0_vec,varNames[1:end])
ax = gca()
for i=1:(num_series-1)
    j = deepcopy(i)
    if i>3
        j = i+1
    end
    ax.get_yticklabels()[i].set_color(color_vec[j])
end
ax.spines["top"].set_color("none")
ax.spines["left"].set_color("none")
ax.spines["right"].set_color("none")
xticks(collect(1965:5:2015),["";"1970";"";"1980";"";"1990";"";"2000";"";"2010";""])
tight_layout()
savefig("/Users/$(usr)/<Insert rest of path to storage folder for figures>/break_plot_joy_int_no_FF.pdf")


#######################################################################
# Calculate AR break probability by date for each variable
#######################################################################
println("Calculating posterior AR break probabilities by date for each series...")
mbreaks_AR = zeros(Tmax,k)
for i=1:k
    series_name = varNames[i]
    casual_name = casualNames[i]
    println("$(i) of $(k) - $(casual_name)")

    T = Int64(T_vec[i])
    qj = q_sampled_meta[:,1:T,:,i]
    
    # Matrix containing all breaks
    breaks = zeros(T,M)
    for iii=1:T
        for j=1:M
            if sum(qj[2:end-1,iii,j]).>0
                breaks[iii,j] = 1
            end
        end
    end
    
    if T==Tmax
        mbreaks_AR[:,i] = mean(breaks,dims=2)
    else
        mbreaks_AR[(Tmax-T+1):end,i] = mean(breaks,dims=2)
    end
end
println("")

#######################################################################
# Create ridge plot for all AR breaks (no FF rate)
# Figure 2 in paper
#######################################################################
figure(figsize=(10,8))
for i in series_no_FF
    if i<=3
        y = mbreaks_AR[:,num_series-i+1]
    elseif (i==5) || (i==6)
        y = mbreaks_AR[:,num_series-i+2]
    elseif i>6
        y = mbreaks_AR[:,num_series-i+1]
    end
    #y = mbreaks[:,i]
    if i>4
        j=i-1
    else
        j=deepcopy(i)
    end
    y0 = ones(Tmax)*j*displace
    y1 = y .+ j*displace
    #k = mod(num_series,i)
    c = color_vec[i]
    fill_between(dates,y0,y1,lw=1,facecolor=c,zorder=num_series-j+1,alpha=0.7,edgecolor="k")
end
xlim([dates[1];dates[end]])
y0_vec = zeros(num_series)

for i=1:(num_series-1)
    y0_vec[i] = i*displace
    j=i-1
end
yticks(y0_vec,casualNamesNoFF[end:-1:1])

#yticks(y0_vec,varNames[1:end])
ax = gca()
for i=1:(num_series-1)
    j = deepcopy(i)
    if i>3
        j = i+1
    end
    ax.get_yticklabels()[i].set_color(color_vec[j])
end
ax.spines["top"].set_color("none")
ax.spines["left"].set_color("none")
ax.spines["right"].set_color("none")
xticks(collect(1965:5:2015),["";"1970";"";"1980";"";"1990";"";"2000";"";"2010";""])
tight_layout()
savefig("/Users/$(usr)/<Insert rest of path to storage folder for figures>/break_plot_joy_AR_no_FF.pdf")

#######################################################################
# Calculate variance break probability by date for each variable
#######################################################################
println("Calculating posterior variance break probabilities by date for each series...")
mbreaks_var = zeros(Tmax,k)
for i=1:k
    series_name = varNames[i]
    casual_name = casualNames[i]
    println("$(i) of $(k) - $(casual_name)")
    T = Int64(T_vec[i])
    qj = q_sampled_meta[:,1:T,:,i]
    # Matrix containing all breaks
    breaks = zeros(T,M)
    for iii=1:T
        for j=1:M
            if sum(qj[end,iii,j]).>0
                breaks[iii,j] = 1
            end
        end
    end
    
    if T==Tmax
        mbreaks_var[:,i] = mean(breaks,dims=2)
    else
        mbreaks_var[(Tmax-T+1):end,i] = mean(breaks,dims=2)
    end
end
println("")

#######################################################################
# Create ridge plot for all variance breaks (no FF rate)
# Figure 3 in paper
#######################################################################
figure(figsize=(10,8))
for i in series_no_FF
    if i<=3
        y = mbreaks_var[:,num_series-i+1]
    elseif (i==5) || (i==6)
        y = mbreaks_var[:,num_series-i+2]
    elseif i>6
        y = mbreaks_var[:,num_series-i+1]
    end
    #y = mbreaks[:,i]
    if i>4
        j=i-1
    else
        j=deepcopy(i)
    end
    y0 = ones(Tmax)*j*displace
    y1 = y .+ j*displace
    #k = mod(num_series,i)
    c = color_vec[i]
    fill_between(dates,y0,y1,lw=1,facecolor=c,zorder=num_series-j+1,alpha=0.7,edgecolor="k")
end
xlim([dates[1];dates[end]])
y0_vec = zeros(num_series)

for i=1:(num_series-1)
    y0_vec[i] = i*displace
    j=i-1
end
yticks(y0_vec,casualNamesNoFF[end:-1:1])

#yticks(y0_vec,varNames[1:end])
ax = gca()
for i=1:(num_series-1)
    j = deepcopy(i)
    if i>3
        j = i+1
    end
    ax.get_yticklabels()[i].set_color(color_vec[j])
end
ax.spines["top"].set_color("none")
ax.spines["left"].set_color("none")
ax.spines["right"].set_color("none")
xticks(collect(1965:5:2015),["";"1970";"";"1980";"";"1990";"";"2000";"";"2010";""])
tight_layout()
savefig("/Users/$(usr)/<Insert rest of path to storage folder for figures>/break_plot_joy_var_no_FF.pdf")


#######################################################################
# Plot posterior unconditional mean with credible interval for 
# growth series
# Figure 4 in paper
#######################################################################
println("Plotting unconditional mean for each growth series (figure 4)")

growthNames = ["GDPC1";"PCECC96";"UNRATE"];
growthTitles = ["Real GDP Growth";"Real PCE Growth";"Unemployment Rate"]
figure(figsize=(10,7))
for k=1:length(growthNames)

    series_name = growthNames[k]
    casual_name = growthTitles[k]
    println("$(k) of $(length(growthNames)) - $(casual_name)")

    
    ###########################################################
    # Restrict to stationary draws
    ###########################################################
    i = findall(x->x!=0,varNames.==series_name)[1]
    T = Int64(T_vec[i])
    B = B_sampled_meta[:,1:T,:,i]
    AR = AR_sampled_meta[1:T,:,i]
    stationary_ind = trues(M)
    for i=1:M
        flag = 0    
        for j=1:T
            F = zeros(k_pot,k_pot)
            F[1,:] = B[2:end,j,i][:]
            F[2:end,1:end-1] = Matrix{Float64}(I, (k_pot-1), (k_pot-1))
           max_eig = maximum(abs.(eigvals(F)[1]))
            if max_eig>0.995
                flag=1
            end
        end
        if flag==1
            stationary_ind[i] = false
        end
    end

    B = B[:,:,stationary_ind]
    G = length(findall(x->x!=0,stationary_ind.==true))


    RGDP_growth = zeros(T,G)
    for i=1:G
        B_temp = B[:,:,i]
        int = B_temp[1,:]
        AR = sum(B_temp[2:end,:],dims=1)[:]
        RGDP_growth[:,i] = int ./ (1 .- AR)
    end
    if k<3
        RGDP_growth = 100 .* ((1 .+ (RGDP_growth./100)).^4 .- 1)
    end

    low95 = zeros(T)
    high95 = zeros(T)
    avg = zeros(T)
    for i=1:T
        low95[i] = quantile(RGDP_growth[i,:],0.16)
        high95[i] = quantile(RGDP_growth[i,:],0.84)
        avg[i] = median(RGDP_growth[i,:])
    end


    dates_all = collect(1959:0.25:2018.25)
    dates_all = dates_all[k_pot+1:end]
    if T==Tmax
        dates = deepcopy(dates_all)
    else
        dates =  dates_all[(Tmax-T+1):end]
    end
    
    if k==1
        subplot2grid((2,4), (0,0), colspan=2)
    elseif k==2 
        subplot2grid((2,4), (0,2), colspan=2)
    else
        subplot2grid((2,4), (1,1), colspan=2)
    end
    
    plot(dates,avg,label="Posterior Median")
    title("$(growthTitles[k])")
    fill_between(dates,low95,high95,color="C0",alpha=0.1,label="68% CI")
    xlim([dates_all[1];dates_all[end]])
    ylabel("Annualized Rate")
    legend(loc="best")
    if k<3
        ylim([1.5;4.0])
        yticks([1.5;2;2.5;3;3.5;4],[1.5;2;2.5;3;3.5;4])
        ylabel("Annualized Rate")
    else
        ylabel("Percent")
    end
end
tight_layout()
savefig("/Users/$(usr)/<Insert rest of path to storage folder for figures>/all_growth.pdf")

#######################################################################
# Plot posterior unconditional mean with credible interval for 
# growth series, including the raw data as well
# Figure 5 in the paper
#######################################################################
println("")
println("Plotting unconditional mean for each growth series with raw data (figure 5)")

growthNames = ["GDPC1";"PCECC96";"UNRATE"];
growthTitles = ["Real GDP Growth";"Real PCE Growth";"Unemployment Rate"]
B = 0

y_lim_growth = zeros(2)
for k=1:2
    series_name = growthNames[k]
    Y = convert(Array{Float64,1},readdlm("Break_data/$(series_name).csv",',')[2:end,2])
    Y = 100 .* ((1 .+ (Y./100)).^4 .- 1)
    
    if minimum(Y)-1 < y_lim_growth[1]
        y_lim_growth[1] = floor(minimum(Y)-1)
    end
    if maximum(Y)+1 > y_lim_growth[2]
        y_lim_growth[2] = ceil(maximum(Y)+1)
    end
end
ylab_growth = collect(y_lim_growth[1]:5:y_lim_growth[2])

figure(figsize=(10,7))
for k=1:length(growthNames)
    series_name = growthNames[k]
    casual_name = growthTitles[k]
    println("$(k) of $(length(growthNames)) - $(casual_name)")

    Y = convert(Array{Float64,1},readdlm("Break_data/$(series_name).csv",',')[2:end,2])
    
    ###########################################################
    # Restrict to stationary draws
    ###########################################################
    i = findall(x->x!=0,varNames.==series_name)[1]
    T = Int64(T_vec[i])
    B = B_sampled_meta[:,1:T,:,i]
    AR = AR_sampled_meta[1:T,:,i]
    stationary_ind = trues(M)
    for i=1:M
        flag = 0    
        for j=1:T
            F = zeros(k_pot,k_pot)
            F[1,:] = B[2:end,j,i][:]
            F[2:end,1:end-1] = Matrix{Float64}(I, (k_pot-1), (k_pot-1))
           max_eig = maximum(abs.(eigvals(F)[1]))
            if max_eig>0.995
                flag=1
            end
        end
        if flag==1
            stationary_ind[i] = false
        end
    end

    B = B[:,:,stationary_ind]
    G = length(findall(x->x!=0,stationary_ind.==true))


    RGDP_growth = zeros(T,G)
    for i=1:G
        B_temp = B[:,:,i]
        int = B_temp[1,:]
        AR = sum(B_temp[2:end,:],dims=1)[:]
        RGDP_growth[:,i] = int ./ (1 .- AR)
    end
    if k<3
        RGDP_growth = 100 .* ((1 .+ (RGDP_growth./100)).^4 .- 1)
        Y = 100 .* ((1 .+ (Y./100)).^4 .- 1)
    end

    low95 = zeros(T)
    high95 = zeros(T)
    avg = zeros(T)
    for i=1:T
        low95[i] = quantile(RGDP_growth[i,:],0.16)
        high95[i] = quantile(RGDP_growth[i,:],0.84)
        avg[i] = median(RGDP_growth[i,:])
    end


    dates_all = collect(1959:0.25:2018.25)
    dates_all = dates_all[k_pot+1:end]
    if T==Tmax
        dates = deepcopy(dates_all)
    else
        dates =  dates_all[(Tmax-T+1):end]
    end
    
    if k==1
        subplot2grid((2,4), (0,0), colspan=2)
    elseif k==2 
        subplot2grid((2,4), (0,2), colspan=2)
    else
        subplot2grid((2,4), (1,1), colspan=2)
    end
    
    plot(dates,Y[(k_pot+1):end],color="C3",label="Raw Data")
    plot(dates,avg,label="Posterior Median",linestyle="--")
    title("$(growthTitles[k])")
    fill_between(dates,low95,high95,color="C0",alpha=0.2,label="68% CI")
    xlim([dates_all[1];dates_all[end]])
    if k==1
        legend(loc="best")
    end
    if k<3
        ylim(y_lim_growth)
        yticks(ylab_growth,ylab_growth)
        ylabel("Annualized Rate")
    else
        ylabel("Percent")
    end  
end
tight_layout()
savefig("/Users/$(usr)/<Insert rest of path to storage folder for figures>/all_growth_with_data.pdf")


#######################################################################
# Plot posterior sum of AR coefficients for inflation series
# Figure 6 in the paper
#######################################################################
println("")
println("Plotting sum of AR coefficients for each inflation series (figure 6)")

piNames = varNames[1:3]
casualPiNames = casualNames[1:3];
figure(figsize=(6,10))
for k=1:length(piNames)

    series_name = piNames[k]
    casual_name = casualPiNames[k]
    println("$(k) of $(length(growthNames)) - $(casual_name)")

    T = Int64(T_vec[k])
    B = B_sampled_meta[:,1:T,:,k]
    G = deepcopy(M)
    
    AR_mat = zeros(T,G)
    for i=1:G
        B_temp = B[:,:,i]
        int = B_temp[1,:]
        AR_mat[:,i] = sum(B_temp[2:end,:],dims=1)[:]
    end


        low95 = zeros(T)
        high95 = zeros(T)
        avg = zeros(T)
        for i=1:T
            low95[i] = quantile(AR_mat[i,:],0.16)
            high95[i] = quantile(AR_mat[i,:],0.84)
            avg[i] = median(AR_mat[i,:])
        end

    dates_all = collect(1959:0.25:2018.25)
    dates_all = dates_all[k_pot+1:end]
    if T==Tmax
        dates = deepcopy(dates_all)
    else
        dates =  dates_all[(Tmax-T+1):end]
    end


    subplot(3,1,k)
    title("$(casualPiNames[k])")
    plot(dates,avg,label="Posterior Median")
    fill_between(dates,low95,high95,color="C0",alpha=0.1,label="68% CI")
    xlim([dates_all[1];dates_all[end]])
    ylim([0;1])
    if k==1
        legend(loc="best")
    end
end
tight_layout()
savefig("/Users/$(usr)/<Insert rest of path to storage folder for figures>/AR_sum_pi.pdf")


#######################################################################
# Plot posterior unconditional mean of inflation series
# Figure 7 in the paper
#######################################################################
println("")
println("Plotting unconditional mean of each inflation series (figure 7)")

figure(figsize=(6,10))
for k=1:length(piNames)
    series_name = piNames[k]
    casual_name = casualPiNames[k]
    println("$(k) of $(length(piNames)) - $(casual_name)")

    ###########################################################
    # Restrict to stationary draws
    ###########################################################
    T = Int64(T_vec[k])
    B = B_sampled_meta[:,1:T,:,k]
    AR = AR_sampled_meta[1:T,:,k]
    stationary_ind = trues(M)
    for i=1:M
        flag = 0    
        for j=1:T
            F = zeros(k_pot,k_pot)
            F[1,:] = B[2:end,j,i][:]
            F[2:end,1:end-1] = Matrix{Float64}(I, (k_pot-1), (k_pot-1))
            max_eig = maximum(abs.(eigvals(F)[1]))
             if max_eig>0.995
                    flag=1
             end
        end
        if flag==1
            stationary_ind[i] = false
        end
    end
    B = B[:,:,stationary_ind]
    G = length(findall(x->x==true,stationary_ind))


    RGDP_growth = zeros(T,G)
    for i=1:G
        B_temp = B[:,:,i]
        int = B_temp[1,:]
        AR = sum(B_temp[2:end,:],dims=1)[:]
        RGDP_growth[:,i] = int./(1 .- AR)
    end
    RGDP_growth = 100 .* ((1 .+ (RGDP_growth./100)).^4 .- 1)


    low95 = zeros(T)
    high95 = zeros(T)
    avg = zeros(T)
    for i=1:T
        low95[i] = quantile(RGDP_growth[i,:],0.16)
        high95[i] = quantile(RGDP_growth[i,:],0.84)
        avg[i] = median(RGDP_growth[i,:])
    end

    dates_all = collect(1959:0.25:2018.25)
    dates_all = dates_all[k_pot+1:end]
    if T==Tmax
        dates = deepcopy(dates_all)
    else
        dates =  dates_all[(Tmax-T+1):end]
    end


    subplot(3,1,k)
    title("$(casualPiNames[k])")
    plot(dates,avg,label="Posterior Median")
    fill_between(dates,low95,high95,color="C0",alpha=0.1,label="68% CI")
    xlim([dates_all[1];dates_all[end]])
    ylim([1;9])
    ylabel("Annualized Rate")
    if k==1
        legend(loc="best")
    end
end
tight_layout()
savefig("/Users/$(usr)/<Insert rest of path to storage folder for figures>/mean_pi.pdf")


#######################################################################
# Plot posterior unconditional mean of inflation series with raw data
# Figure 8 in the paper
#######################################################################
println("")
println("Plotting unconditional mean of each inflation series with raw data (figure 8)")

y_lim_pi = zeros(2)
for k=1:3
    series_name = piNames[k]
    Y = convert(Array{Float64,1},readdlm("Break_data/$(series_name).csv",',')[2:end,2])
    Y = 100 .* ((1 .+ (Y./100)).^4 .- 1)
    
    if minimum(Y)-1 < y_lim_pi[1]
        y_lim_pi[1] = floor(minimum(Y)-1)
    end
    if maximum(Y)+1 > y_lim_pi[2]
        y_lim_pi[2] = ceil(maximum(Y)+1)
    end
end
ylab_pi = collect(y_lim_pi[1]:5:y_lim_pi[2])

figure(figsize=(6,10))
for k=1:length(piNames)
    series_name = piNames[k]
    casual_name = casualPiNames[k]
    println("$(k) of $(length(piNames)) - $(casual_name)")

    Y = convert(Array{Float64,1},readdlm("Break_data/$(series_name).csv",',')[2:end,2])
    Y = 100 .* ((1 .+ (Y./100)).^4 .- 1)
    
    ###########################################################
    # Restrict to stationary draws
    ###########################################################
    T = Int64(T_vec[k])
    B = B_sampled_meta[:,1:T,:,k]
    AR = AR_sampled_meta[1:T,:,k]
    stationary_ind = trues(M)
    for i=1:M
        flag = 0    
        for j=1:T
            F = zeros(k_pot,k_pot)
            F[1,:] = B[2:end,j,i][:]
            F[2:end,1:end-1] = Matrix{Float64}(I, (k_pot-1), (k_pot-1))
            max_eig = maximum(abs.(eigvals(F)[1]))
             if max_eig>0.995
                    flag=1
             end
        end
        if flag==1
            stationary_ind[i] = false
        end
    end
    B = B[:,:,stationary_ind]
    G = length(findall(x->x==true,stationary_ind))


    RGDP_growth = zeros(T,G)
    for i=1:G
        B_temp = B[:,:,i]
        int = B_temp[1,:]
        AR = sum(B_temp[2:end,:],dims=1)[:]
        RGDP_growth[:,i] = int./(1 .- AR)
    end
    RGDP_growth = 100 .* ((1 .+ (RGDP_growth./100)).^4 .- 1)


    low95 = zeros(T)
    high95 = zeros(T)
    avg = zeros(T)
    for i=1:T
        low95[i] = quantile(RGDP_growth[i,:],0.16)
        high95[i] = quantile(RGDP_growth[i,:],0.84)
        avg[i] = median(RGDP_growth[i,:])
    end

    dates_all = collect(1959:0.25:2018.25)
    dates_all = dates_all[k_pot+1:end]
    if T==Tmax
        dates = deepcopy(dates_all)
    else
        dates =  dates_all[(Tmax-T+1):end]
    end


    subplot(3,1,k)
    title("$(casualPiNames[k])")
    plot(dates,Y[(k_pot+1):end],color="C3",label="Raw Data")
    plot(dates,avg,label="Posterior Median",linestyle="--")
    fill_between(dates,low95,high95,color="C0",alpha=0.2,label="68% CI")
    xlim([dates_all[1];dates_all[end]])
    if k==1
        legend(loc="best")
    end
    ylim(y_lim_pi)
    yticks(ylab_pi,ylab_pi)
    ylabel("Annualized Rate")
end

tight_layout()
savefig("/Users/$(usr)/<Insert rest of path to storage folder for figures>/mean_pi_with_data.pdf")


#######################################################################
# Plot posterior unconditional variance of each series with raw data
# Figure 9 in the paper
#######################################################################
println("")
println("Calculating unconditional variance of each each series. This will take a while.")

unc_sd_median = zeros(Tmax,9)
unc_sd_low = zeros(Tmax,9)
unc_sd_high = zeros(Tmax,9)

for k=1:length(varNames)
    series_name = varNames[k]
    casual_name = casualNames[k]
    println("$(k) of $(length(varNames)) - $(casual_name)")
    
    ###########################################################
    # Restrict to stationary draws
    ###########################################################
    T = Int64(T_vec[k])
    B = B_sampled_meta[:,1:T,:,k]
    H = H_sampled_meta[1:T,:,k]
    AR = AR_sampled_meta[1:T,:,k]
    stationary_ind = trues(M)

    for i=1:M
        flag = 0    
        for j=1:T
            F = zeros(k_pot,k_pot)
            F[1,:] = B[2:end,j,i][:]
            F[2:end,1:end-1] = Matrix{Float64}(I, (k_pot-1), (k_pot-1))
            max_eig = maximum(abs.(eigvals(F)[1]))
             if max_eig>0.995
                    flag=1
             end
        end
        if flag==1
            stationary_ind[i] = false
        end
    end

    B = B[:,:,stationary_ind]
    H = H[:,stationary_ind]
    G = length(findall(x->x==true,stationary_ind))

    unc_var = zeros(T,G)
    F = zeros(k_pot,k_pot)
    F[2:end,1:end-1] = Matrix{Float64}(I, (k_pot-1), (k_pot-1))
    Q = zeros(k_pot,k_pot)
    for i=1:G
        for j=1:T
            F[1,:] = B[2:end,j,i][:]
            Q[1,1] = 1 ./ H[j,i]
            V = dlyap(F,Q)
            if V[1]>0
                unc_var[j,i] = V[1]
            end
        end
    end

    println("")
    σ = sqrt.(unc_var)
    low95 = zeros(T)
    high95 = zeros(T)
    avg = zeros(T)
    for i=1:T
        low95[i] = quantile(σ[i,:],0.16)
        high95[i] = quantile(σ[i,:],0.84)
        avg[i] = median(σ[i,:])
    end
    
    if T==Tmax
        unc_sd_median[:,k] = avg
        unc_sd_low[:,k] = low95
        unc_sd_high[:,k] = high95
    else
        unc_sd_median[(Tmax-T+1):end,k] = avg
        unc_sd_low[(Tmax-T+1):end,k] = low95
        unc_sd_high[(Tmax-T+1):end,k] = high95
    end
end

println("")
println("Plotting unconditional variance of each each series with raw data (figure 9)")

empNames = varNames[end-1:end]
casualEmpNames = casualNames[end-1:end]

figure(figsize=(15,10.5))
for k=1:9
    if k==4
        continue
    end
    series_name = varNames[k]
    casual_name = casualNames[k]
    println("$(k) of $(length(varNames)) - $(casual_name)")

    Y = convert(Array{Float64,1},readdlm("Break_data/$(series_name).csv",',')[2:end,2])
    #Y = 100 .* ((1 .+ (Y./100)).^4 .- 1)
    T = Int64(T_vec[k])
    dates_all = collect(1959:0.25:2018.25)
    dates_all = dates_all[k_pot+1:end]
    if T==Tmax
        dates = deepcopy(dates_all)
        avg = unc_sd_median[:,k]
        low95 = unc_sd_low[:,k]
        high95 = unc_sd_high[:,k]
    else
        dates =  dates_all[(Tmax-T+1):end]
        avg = unc_sd_median[(Tmax-T+1):end,k]
        low95 = unc_sd_low[(Tmax-T+1):end,k]
        high95 = unc_sd_high[(Tmax-T+1):end,k]
    end

    if T==Tmax
        dates = deepcopy(dates_all)
    else
        dates =  dates_all[(Tmax-T+1):end]
    end
    
    if k<=3
        subplot(3,3,k)
    elseif k==4
        subplot(3,3,9)
    else
        subplot(3,3,k-1)
    end
    ax = gca()
    title("$(casualNames[k])")
    plot(dates,(Y[k_pot+1:end]),color="C3")
    #plot(dates,avg,color="C0",linestyle="--",label="Posterior Median") 
    #fill_between(dates,low95,high95,color="C0",alpha=0.1,label="68% CI")
    xlim([dates_all[1];dates_all[end]])
    #ylabel("Absolute Difference of Data",color="C3")

    ax2 = ax.twinx() 
    ax2.plot(dates[end-1:end].+10,(Y[end-1:end]),color="C3",label="Raw Data")
    ax2.plot(dates,avg,color="C0",linestyle="--",label="Posterior Median") 
    ax2.fill_between(dates,low95,high95,color="C0",alpha=0.1,label="68% CI")
    yticks(color="C0")
    ylabel("Unconditional SD",color="C0")
    
    if k==1
        legend(loc="best")
    end
    xlim([dates_all[1];dates_all[end]])

end
tight_layout()
savefig("/Users/$(usr)/<Insert rest of path to storage folder for figures>/SD_all_but_FF_dual_axes.pdf")